@@ -20,7 +20,7 @@
  *
  */
 
-static char const rcsid[]="$Id: toshset.cc,v 1.45 2008-03-21 02:27:22 schwitrs Exp $";
+static char const rcsid[]="$Id: toshset.cc,v 1.50 2009-06-04 22:05:35 schwitrs Exp $";
 static char const rcsversion[]="$State: Exp $";
 
 
@@ -38,11 +38,15 @@ static char const rcsversion[]="$State: Exp $";
 #include<signal.h>
 #include<paths.h>
 #include<pwd.h>
-#include<features.h>
 #include <termios.h>
 #include<sys/ioctl.h>
-#include<sys/vt.h>
-#include<sys/kd.h>
+#ifdef LINUX
+#  include<features.h>
+#  include<sys/vt.h>
+#  include<sys/kd.h>
+#else /* LINUX */
+#  include <term.h>
+#endif
 #include <iostream>
 #include <streambuf>
 #include <iomanip>
@@ -209,13 +213,16 @@ SciFeature::action(const char **s) const
        return 1;
    }
  // try setting indexed setting of option.
- int i = atoi(*s);
- if ( ret<0 && strlen(*s)==1 && isdigit(**s) && i>=0 && i<values.size() ) {
-   reg.ecx =values[i]->sciCode;
-   reg.edx = reg.esi = reg.edi = 0;
-   ret = SciSet( &reg );
-   if ( ret == SCI_SUCCESS ) 
-     return 1;
+ if ( ret<0 ) {
+   errno=0;
+   int i = strtol(*s,(char **)NULL,10);
+   if ( errno==0 && i>=0 && i<values.size() ) {
+     reg.ecx =values[i]->sciCode;
+     reg.edx = reg.esi = reg.edi = 0;
+     ret = SciSet( &reg );
+     if ( ret == SCI_SUCCESS ) 
+       return 1;
+   }
  }
  if ( ret>=0 ) {
    cerr << "SciFeature:action: error setting " << name << '\n';
@@ -783,7 +790,7 @@ struct WirelessFeature : public HciFeature {
   int mode;
   WirelessFeature(int mode,
 		  const char* name) :
-    HciFeature(0x56,name), mode(mode) {}
+    HciFeature(HCI_WIRELESS,name), mode(mode) {}
   virtual int action(const char**) const;
   virtual int query(OStringStream& os) const;
 };
@@ -858,7 +865,7 @@ WirelessFeature::query(OStringStream& os) const
 
 struct BlueToothFeature : public HciFeature {
   BlueToothFeature(const char* name) :
-    HciFeature(0x56,name) {}
+    HciFeature(HCI_WIRELESS,name) {}
   virtual int action(const char**) const;
   virtual int query(OStringStream& os) const;
 };
@@ -1009,6 +1016,122 @@ BlueToothFeature::query(OStringStream& os) const
  return ret;
 } /* BlueToothFeature::query */
 
+struct ThreeGRFFeature : public HciFeature {
+  ThreeGRFFeature(const char* name) :
+    HciFeature(HCI_WIRELESS,name) {}
+  virtual int action(const char**) const;
+  virtual int query(OStringStream& os) const;
+};
+
+int
+ThreeGRFFeature::action(const char** c) const
+{
+ SMMRegisters reg;
+ reg.eax = HCI_GET;
+ reg.ebx = hciMode;
+ reg.ecx = 0;
+ reg.edx = 0;
+ int ret=0;
+ ret = HciFunction( &reg );
+ if (  ret!=0 ) {
+   //   cerr << "error querying 3g modem\n";
+   return ret;
+ }
+ if (!(reg.ecx & 0x0f)) {
+   cerr << "3g modem unavailable\n";
+   return ret;
+ }
+ reg.eax = HCI_GET;
+ reg.ebx = hciMode;
+ reg.ecx = 0;
+ reg.edx = 1;
+ ret=0;
+ ret = HciFunction( &reg );
+ if ( ret!=0 ) {
+   cerr << "error checking 3g modem status\n";
+   return ret;
+ }
+ if(!(reg.ecx & 0x1)) {
+   cerr << "wireless switch is off\n";
+   return ret;
+ }
+
+ if ( (String(*c) == "on") ||
+      (String(*c) == "1")    ) { // turn on
+
+   reg.eax = HCI_SET;
+   reg.ebx = hciMode;
+   reg.ecx = 1;
+   reg.edx = 0x2000;
+   ret=0;
+   ret = HciFunction( &reg );
+   if ( ret!=0 ) {
+     cerr << "error activating 3g modem device\n";
+     return ret;
+   }
+ }
+ if ( (String(*c) == "off") ||
+      (String(*c) == "0") )  { // turn off
+   reg.eax = HCI_SET;
+   reg.ebx = hciMode;
+   reg.ecx = 0;
+   reg.edx = 0x2000;
+   ret=0;
+   ret = HciFunction( &reg );
+   if ( ret!=0 ) {
+     cerr << "error deactivating 3g modem device\n";
+     return ret;
+   }
+   return ret;
+ }
+return ret;
+} /* ThreeGRFFeature::action */
+
+int
+ThreeGRFFeature::query(OStringStream& os) const
+{
+ SMMRegisters reg;
+ reg.eax = HCI_GET;
+ reg.ebx = hciMode;
+ reg.ecx = 0;
+ reg.edx = 0;
+ int ret=0;
+ ret = HciFunction( &reg );
+ if (  ret!=0 ) {
+   //   cerr << "error querying 3g modem\n";
+   return ret;
+ }
+ if (!(reg.ecx & 0x0f)) { //At the modem this is the same as bluetooth
+	                  //I don't know how to check for 3g modem presence
+   cerr << "3g modem unavailable\n";
+   return ret;
+ }
+ reg.eax = HCI_GET;
+ reg.ebx = hciMode;
+ reg.ecx = 0;
+ reg.edx = 1;
+ ret=0;
+ ret = HciFunction( &reg );
+ if ( ret!=0 ) {
+   cerr << "error checking 3g switch status\n";
+   return ret;
+ }
+ os << "3g modem: ";
+ if(!(reg.ecx & 0x1)) {
+   os << "wireless switch is off\n";
+   return ret;
+ }
+ if(!(reg.ecx & 0x2000)) {
+   os << "off";
+   return ret;
+ }
+
+ os << "on";
+ return ret;
+} /* ThreeGRFFeature::query */
+
+
+
 struct VideoFeature : public HciFeature {
   VideoFeature(unsigned short hciMode,const char* name) :
     HciFeature(hciMode,name) {}
@@ -1296,9 +1419,12 @@ LCDFeature::query(OStringStream& os) const
      case HCI_1024_768: os << "1024x768, "; break;
      case HCI_1024_600: os << "1024x600, "; break;
      case HCI_800_480 : os << " 800x480, "; break;
-     case HCI_1400_1050 : os << " 1400_1050, "; break;
+     case HCI_1400_1050 : os << " 1400x1050, "; break;
      case HCI_1600_1200 : os << " 1600x1200, "; break;
      case HCI_1280_600  : os << " 1280x600, "; break;
+     case HCI_1280_800  : os << " 1280x800, "; break;
+     case HCI_1440_900  : os << " 1440x900, "; break;
+     case HCI_1920_1200 : os << " 1920x1200, "; break;
      default: os << "resolution (" << ((reg.ecx & 0xff00)>>8 ) << ") unknown";
    }
    switch ( reg.ecx & 0xff ) {
@@ -1517,17 +1643,11 @@ main(      int   argc,
 {
  int version,bios;
 
- ///* this program *must* be run as root */
- //if (getuid()) {
- //  cerr << argv[0] << ": must be run as root.\n";
- //  return 1;
- //}
-
- if (ioperm(0xb2, 1, 1)) {
-   cerr << argv[0] << ": can't get I/O permissions.\n" 
-	<< "\tPlease run as root.\n";
-   return 1;
- }
+// if (ioperm(0xb2, 1, 1)) {
+//   cerr << argv[0] << ": can't get I/O permissions.\n" 
+//	<< "\tRead/write permission to /dev/toshiba is required.\n";
+//   return 1;
+// }
 
  // should be use /dev/toshiba or direct calls to the BIOS
  detAccessMode();
@@ -1621,18 +1741,22 @@ main(      int   argc,
  features.append(&backlightFeature);
 
  HciFeature trBacklightFeature(HCI_TR_BACKLIGHT,"transreflective mode");
- trBacklightFeature.addValue( "off"  , HCI_DISABLE   , "off" );
- trBacklightFeature.addValue( "on"   , HCI_ENABLE   , "on" );
+ trBacklightFeature.addValue( "off"  , HCI_ENABLE   , "off" );
+ trBacklightFeature.addValue( "on"   , HCI_DISABLE   , "on" );
  features.append(&trBacklightFeature);
 
  HciFeature fanFeature(HCI_FAN,"fan");
  fanFeature.addValue( "off"  , HCI_DISABLE   , "off" );
- fanFeature.addValue( "1/4" , HCI_FAN_LOW2  , "1/4" );
- fanFeature.addValue( "2/4"  , HCI_FAN_LOW3   , "2/4" );
- fanFeature.addValue( "3/4" , HCI_FAN_HIGH2  , "3/4" );
- fanFeature.addValue( "high" , HCI_FAN_HIGH  , "high" );
  fanFeature.addValue( "on"   , HCI_ENABLE   , "on" );
+ fanFeature.addValue( "fan1" , HCI_FAN_FAN1  , "fan1" );
+ fanFeature.addValue( "fan2" , HCI_FAN_FAN2  , "fan2" );
+ fanFeature.addValue( "1/4" , HCI_FAN_LOW2  , "1/4" );
+ fanFeature.addValue( "3/4" , HCI_FAN_HIGH2  , "3/4" );
+ fanFeature.addValue( "2/4"  , HCI_FAN_LOW3   , "2/4" );
+ fanFeature.addValue( "high1" , HCI_FAN_HIGH1  , "high1" );
+ fanFeature.addValue( "high" , HCI_FAN_HIGH  , "high" );
  fanFeature.addValue( "low" , HCI_FAN_LOW  , "low" );
+ fanFeature.addValue( "1/8" , HCI_FAN_LOW4  , "1/8" );
  features.append(&fanFeature);
 
  HciFeature selectBayFeature(HCI_SELECT_STATUS,"select bay");
@@ -1891,7 +2015,7 @@ main(      int   argc,
  bootFeature.addValue( "cdhdfd" , 5         , "CDROM->hard disk->floppy");
  features.append(&bootFeature);
 
- HciFeature wirelessFeature(0x56,"wireless support");
+ HciFeature wirelessFeature(HCI_WIRELESS,"wireless support");
  wirelessFeature.addValue( "" , 0 , "not present");
  wirelessFeature.addValue( "" , 0xf , "present");
  features.append(&wirelessFeature);
@@ -1902,6 +2026,10 @@ main(      int   argc,
  BlueToothFeature blueToothFeature("bluetooth");
  features.append(&blueToothFeature);
 
+ ThreeGRFFeature threeGRFFeature("3g");
+ features.append(&threeGRFFeature);
+
+
  //FIX: need to enumerate these
  HciFeature bootDeviceFeature(HCI_BOOT_DEVICE,"boot device");
  //features.append(&bootDeviceFeature);
@@ -1956,6 +2084,8 @@ main(      int   argc,
 		 &autoOffFeature),
    new ArgSet<1>("-bluetooth","<on|off> enable/disable internal bluetooth",
 		 &blueToothFeature),
+   new ArgSet<1>("-3g","<on|off> enable/disable 3g modem frequency",
+                 &threeGRFFeature),
    new ArgSet<1>("-usblegacy","<enable|disable> USB legacy mode",&legacyUSBFeature),
    new ArgSet<1>("-usbfdd","<enable|disable> USB FDD emulation mode",&USBFDDFeature),
    new ArgSet<1>("-lan","<enable|disable> LAN controller",&LANcontrollerFeature),
